
/*
 * Web Polygraph                        http://www.web-polygraph.org/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. See the COPYRIGHT file for details.
 */

#include "pgl/pgl.h"

#include "xstd/h/math.h"

#include "pgl/PglNumSym.h"
#include "pgl/PglTimeSym.h"
#include "pgl/PglRateSym.h"
#include "pgl/PglBwidthSym.h"
#include "pgl/PglSizeSym.h"


SizeSym::SizeSym(const BigSize &aVal): ExpressionSym("size"), theVal(aVal) {
}

bool SizeSym::isA(const String &type) const {
	return ExpressionSym::isA(type) ||
		type == "Size" || type == "size";
}


SynSym *SizeSym::dupe(const String &type) const {
	if (isA(type))
		return new SizeSym(theVal);
	return ExpressionSym::dupe(type);
}

ExpressionSym *SizeSym::unOper(const String &op) const {
	if (*op == '+')
		return new SizeSym(+theVal);
	else
	if (*op == '-')
		return new SizeSym(-theVal);
	else
		return ExpressionSym::unOper(op);
}


ExpressionSym *SizeSym::bnOper(const String &op, const SynSym &s) const {
	if (op.len() != 1)
		return ExpressionSym::bnOper(op, s);

	if (SizeSym *ss = (SizeSym*)s.clone("size"))
		return operSS(op, ss, s);

	if (TimeSym *ts = (TimeSym*)s.clone("time"))
		return operST(op, ts, s);

	if (RateSym *rs = (RateSym*)s.clone("rate"))
		return operSR(op, rs, s);

	if (BwidthSym *bs = (BwidthSym*)s.clone("bwidth"))
		return operSB(op, bs, s);

	if (NumSym *ns = (NumSym*)s.clone("float"))
		return operSN(op, ns, s);
	
	return ExpressionSym::bnOper(op, s);
}

ExpressionSym *SizeSym::operSS(const String &op, SizeSym *ss, const SynSym &s) const {
	BigSize otherVal = ss->val();
	delete ss;
	
	if (*op == '/')
		checkDenom(otherVal != 0);

	switch (*op) {
		case '+': return new SizeSym(theVal + otherVal);
		case '-': return new SizeSym(theVal - otherVal);
		case '/': return new NumSym(theVal / otherVal);
		default: return ExpressionSym::bnOper(op, s);
	}
}

ExpressionSym *SizeSym::operST(const String &op, TimeSym *ts, const SynSym &s) const {
	Time otherInterval = ts->val();
	delete ts;

	switch (*op) {
		case '/': return new BwidthSym(Bandwidth(theVal, otherInterval));
		default: return ExpressionSym::bnOper(op, s);
	}
}

ExpressionSym *SizeSym::operSR(const String &op, RateSym *rs, const SynSym &s) const {
	double rps = rs->val();
	delete rs;

	switch (*op) {
 		case '*': return new BwidthSym(Bandwidth(theVal*rps, Time::Sec(1)));
		default: return ExpressionSym::bnOper(op, s);
	}
}

ExpressionSym *SizeSym::operSB(const String &op, BwidthSym *bs, const SynSym &s) const {
	Bwidth otherVal = bs->val();
	delete bs;
	
	if (*op == '/')
		checkDenom(otherVal);

	switch (*op) {
		case '/': return new TimeSym(Time::Secd(theVal.byted()/BytesPerSec(otherVal)));
		default: return ExpressionSym::bnOper(op, s);
	}
}

ExpressionSym *SizeSym::operSN(const String &op, NumSym *ns, const SynSym &s) const {
	double otherFactor = ns->val();
	delete ns;
	
	if (*op == '/')
		checkDenom(otherFactor);

	switch (*op) {
		case '*': return new SizeSym(theVal*otherFactor);
		case '/': return new SizeSym(theVal/otherFactor);
		default: return ExpressionSym::bnOper(op, s);
	}
}

ostream &SizeSym::print(ostream &os, const String &) const {
	return os << theVal;
}
